// dbg_wrapper_date.hpp 
//
//////////////////////////////////////////////////////////////////////

// Debug Utility library
//
// Copyright (C) 2003 John Torjo (john@torjo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.

#if !defined(BOOST_DBG_WRAPPER_DATE_HPP)
#define BOOST_DBG_WRAPPER_DATE_HPP

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <boost/date_time/gregorian/gregorian_types.hpp>
#include <iomanip>


namespace boost { namespace dbgutil {

    /* 
        specify the handle class

        Using the type directly (for instance, time_t, directly)
        won't work, since we might have multiple handle types 
        that map to the same underlying type.

        For instance, HWND and HINTERNET are different types of handles,
        but they both map to 'void *'.
    */
    struct date_traits {
        typedef ::boost::gregorian::date raw_type;
    };


    template<>
    struct dbg_wrapper_traits< date_traits, char> 
        : public dbg_wrapper_traits_base< date_traits, char> {

        /* 
            writes ONLY the details about the value
        */
        template< class other_char_type, class traits_type>
        static void write_details( 
                handle val, 
                std::basic_ostream< other_char_type, traits_type> & out) {

            const char * months[] =
            { "jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec" };

            // the details
            //
            // note: we write them as a WORD (that is, no spaces),
            //       so that when they are read from the stream, 
            //       they can just be read as a whole word
            //       (keeping it simple)
            out << std::setw(4) << std::setfill('0') << val.year() << '_'
                << months[ val.month()] << '_'
                << std::setw(2) << std::setfill('0') << val.day();
        }

        /* 
            writes ONLY the value (no details about it)
        */
        template< class other_char_type, class traits_type>
        static void write_val( handle val, std::basic_ostream< other_char_type, traits_type> & out) {
            // note: this could be improved A LOT
            // just by reading/writing the internal value !!!

            long year = val.year(), mon = val.month(), day = val.day();
            out << year << ' ' << mon << ' ' << day;
        }

        /*
            read the value (ignoring the details, if any)
        */
        template< class other_char_type, class traits_type>
        static void read_val( 
                handle &val, 
                std::basic_istream< other_char_type, traits_type> & in) {
            // note: this could be improved A LOT
            // just by reading/writing the internal value !!!
            int year = 0, month = 0, day = 0;

            char c = 0;
            in >> c;
            if ( in) {
                if ( c == prefix() ) {
                    // contains details as well (which we ignore)
                    std::string ignore;
                    in >> year >> month >> day >> ignore;
                }
                else {
                    // no details
                    in.putback( c);
                    in >> year >> month >> day;
                }
            }

            val = handle( year, (::boost::gregorian::greg_month)month, day);
        }
    };


}} // namespaces

#endif // !defined(AFX_TIME_T_WRAPPER_H__5ECB1ADD_9D17_41A3_8E21_5DF960B7646A__INCLUDED_)
