// dbg_wrapper_ptime.hpp 
//
//////////////////////////////////////////////////////////////////////

// Debug Utility library
//
// Copyright (C) 2003 John Torjo (john@torjo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.

#if !defined(BOOST_DBG_WRAPPER_PTIME_HPP)
#define BOOST_DBG_WRAPPER_PTIME_HPP

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <boost/dbgutil/dbg_wrapper_date.hpp>
#include <boost/dbgutil/dbg_wrapper_timeduration.hpp>


namespace boost { namespace dbgutil {

    /* 
        specify the handle class

        Using the type directly (for instance, time_t, directly)
        won't work, since we might have multiple handle types 
        that map to the same underlying type.

        For instance, HWND and HINTERNET are different types of handles,
        but they both map to 'void *'.
    */
    struct ptime_traits {
        typedef ::boost::posix_time::ptime raw_type;
    };


    template<>
    struct dbg_wrapper_traits< ptime_traits, char> 
        : public dbg_wrapper_traits_base< ptime_traits, char> {

        typedef dbg_wrapper_traits< date_traits, char> date_traits_type;
        typedef dbg_wrapper_traits< time_duration_traits, char> time_traits_type;

        /* 
            writes ONLY the details about the value
        */
        template< class other_char_type, class traits_type>
        static void write_details( 
                handle val, 
                std::basic_ostream< other_char_type, traits_type> & out) {

            // the details
            //
            // note: we write them as a WORD (that is, no spaces),
            //       so that when they are read from the stream, 
            //       they can just be read as a whole word
            //       (keeping it simple)

            date_traits_type::write_details( val.date(), out);
            out << '_';
            time_traits_type::write_details( val.time_of_day(), out);
        }

        /* 
            writes ONLY the value (no details about it)
        */
        template< class other_char_type, class traits_type>
        static void write_val( handle val, std::basic_ostream< other_char_type, traits_type> & out) {
            
            // note: this could be improved A LOT
            // just by reading/writing the internal value !!!
            date_traits_type::write_val( val.date(), out);
            out << ' ';
            time_traits_type::write_val( val.time_of_day(), out);
        }


        /*
            read the value (ignoring the details, if any)
        */
        template< class other_char_type, class traits_type>
        static void read_val( 
                handle &val, 
                std::basic_istream< other_char_type, traits_type> & in) {
            // note: this could be improved A LOT
            // just by reading/writing the internal value !!!
            ::boost::gregorian::date d( 1970, ::boost::gregorian::Jan, 1); // dummy
            ::boost::posix_time::time_duration t( 0, 0, 0, 0); // dummy

            char c = 0;
            in >> c;
            if ( in) {
                if ( c == prefix() ) {
                    // contains details as well (which we ignore)
                    std::string ignore;
                    date_traits_type::read_val( d, in);
                    time_traits_type::read_val( t, in);
                    in >> ignore;
                }
                else {
                    // no details
                    in.putback( c);
                    date_traits_type::read_val( d, in);
                    time_traits_type::read_val( t, in);
                }
            }
            val = handle( d, t);
        }

    };


}} // namespaces


#endif // !defined(AFX_TIME_T_WRAPPER_H__5ECB1ADD_9D17_41A3_8E21_5DF960B7646A__INCLUDED_)
